from edmond.plugin import Plugin


class PlusPlugin(Plugin):
    """Plugin to store additional content from other plugins.

    This plugin does not do much on its own, it lets other plugins register
    additional content to compute on the fly when asked, e.g. the Wikipedia
    plugin can store the Web page of the definition they just gave so users
    wanting to know more about a definition can use the "plus" function to get
    the URL to the Web page.

    There can be only one handle registered at one time by target (so by
    channel, user, etc). External plugins use the `add_handler` to set the
    current handler for a target.
    """

    REQUIRED_CONFIGS = ["commands"]

    def __init__(self, bot):
        super().__init__(bot)
        self.handlers = {}

    def on_pubmsg(self, event):
        if not self.should_handle_command(event.arguments[0], no_content=True):
            return False
        self.process_handler(event)
        return True

    def process_handler(self, event):
        if handler := self.handlers.pop(event.target, None):
            handler(event)
        else:
            self.signal_failure(event.target)

    def add_handler(self, target: str, handler):
        self.handlers[target] = handler
